/***************************************************************************
 *   Copyright (C) 2009 by Felipe Kermentz Ferraz Costa                    *
 *   felipekfcosta@gmail.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <iostream>
#include <cstdlib>
#include <math.h>


#include "trobot.h"


#define ROBOT_TURNING 0
#define ROBOT_WALKING 1
#define ROBOT_STOPPED 2
#define PI 3.1416


float walk_distance = 0;
float odometer;
int previous_waypoints = 0; // waypoint counter
int robot_status = ROBOT_STOPPED; 

void robotfunc(void* arg) {
  TRobotClient *Robot = (TRobotClient*) arg;

  switch (robot_status) {
    case ROBOT_STOPPED: {
      Robot->data.speed = 0;
      Robot->data.turn_angle = 0;
      break;
    }
    case ROBOT_WALKING: {
      Robot->data.speed = 1;
      if (walk_distance < Robot->data.odometer - odometer) {
	robot_status = ROBOT_STOPPED;
      }
      break;
    }
    case ROBOT_TURNING: {
      Robot->data.speed = 0;
      if (fabs(Robot->data.turn_angle) < 1) Robot->data.turn_angle = 0;
      if (Robot->data.turn_angle == 0) robot_status = ROBOT_WALKING; // finished turn
      break;
    }
  }
  if (Robot->data.num_waypoints > previous_waypoints) {
    float dx = Robot->data.waypoint[previous_waypoints].x - Robot->data.x; 
    float dz = Robot->data.waypoint[previous_waypoints].z - Robot->data.z; 
    robot_status = ROBOT_TURNING; 
    Robot->data.turn_angle = atan(dx/dz)*180.0/PI;    
    if (dx < 0 && dz < 0) Robot->data.turn_angle += 180;
    if (Robot->data.compass < 0) Robot->data.compass = 360 + Robot->data.compass;
    Robot->data.turn_angle -= Robot->data.compass;
    walk_distance = sqrt(dx*dx + dz*dz);
    odometer = Robot->data.odometer; // stores the current odometer counting, so we know the distance to walk
    previous_waypoints = Robot->data.num_waypoints; // updates the waypoint counter
  }

}

int main(int argc, char *argv[])
{
	// connect to a server
	// TRobotClient(char *robot_name, void(void*) robot_loop_procedure, char* server, int port)
	TRobotClient *mytest = new TRobotClient("go_flag", robotfunc, "127.0.0.1", 4000);
	for (;;); // lock the program: the robot function will now run on a separate thread
	return EXIT_SUCCESS;
}

